<?php
/**
 * @file
 * Contains functions related to retrieving/sending outcomes from/to
 * the consumers outcomes service.
 */

module_load_include('inc', 'lti_tool_provider_outcomes', 'lti_tool_provider_outcomes.connect');

/**
 * Send results to the Consumer.
 *
 * @param string $mode
 *   The scope of results to retrieve.
 * @param string $return_url
 *   To customise return page when URL is accessed.
 */
function lti_tool_provider_outcomes_send_results($mode, $return_url = '') {
  global $user;
  // Consumer_id based on the context.
  $uid = $user->uid;
  $consumer_id = $_SESSION['lti_tool_provider_context_info']['consumer_id'];
  $context_id = $_SESSION['lti_tool_provider_context_info']['context_id'];
  $key = $_SESSION['lti_tool_provider_context_info']['oauth_consumer_key'];
  $resource_link_id = $_SESSION['lti_tool_provider_context_info']['resource_link_id'];
  $bool = TRUE;
  // Get the outcomes to send.
  switch ($mode) {
    case 'user': {
      // For result of current user.
      $outcomes_entities = lti_tool_provider_outcomes_get_outcomes($consumer_id, $context_id, $resource_link_id, $uid);
      break;
    }
    // For all results in current resource.
    case 'resource': {
      $outcomes_entities = lti_tool_provider_outcomes_get_outcomes($consumer_id, $context_id, $resource_link_id);
      break;
    }
    // For all results in current context.
    case 'context': {
      $outcomes_entities = lti_tool_provider_outcomes_get_outcomes_for_context($consumer_id, $context_id);
      break;
    }
  }
  if (isset($outcomes_entities)) {
    $consumer = lti_tool_provider_consumer_load($consumer_id);
    $url = $consumer->lti_tool_provider_consumer_outcomes_url;
    if (!isset($url)) {
      drupal_set_message(t('Unable to send results. No Outcome URL is avaliable', 'error'));
      drupal_goto($return_url);
    }
    else {
      $outcomes_counter = 0;
      $total_outcomes = count($outcomes_entities);
      foreach ($outcomes_entities as $outcome) {
        $resource = lti_tool_provider_outcomes_resource_load($outcome->lti_tool_provider_outcomes_resource_entity_id_fk);
        if ($consumer->lti_tool_provider_consumer_ext) {
          $params = lti_tool_provider_outcomes_construct_params_array_update($outcome->lti_tool_provider_outcomes_result_sourcedid, $outcome->lti_tool_provider_outcomes_score, $resource->lti_tool_provider_outcomes_resource_score_datatype_base_value, lti_tool_provider_outcomes_get_resource_datatype($outcome->lti_tool_provider_outcomes_resource_entity_id_fk, 2));
          $response = lti_tool_provider_outcomes_lti_service_post($params, $consumer->lti_tool_provider_consumer_key, $consumer->lti_tool_provider_consumer_outcomes_url);
          $success = lti_tool_provider_outcomes_decode_ext_xml_reply($response);
        }
        else {
          $response = lti_tool_provider_outcomes_lti_service_xml($consumer, $consumer->lti_tool_provider_consumer_outcomes_url, 1, $outcome->lti_tool_provider_outcomes_result_sourcedid, $outcome->lti_tool_provider_outcomes_score);
          $success = lti_tool_provider_outcomes_decode_xml_reply($response);
        }
        if ($success) {
          // Once sent, update timestamp.
          lti_tool_provider_outcomes_update_timestamp($outcome);
          $outcomes_counter++;
        }
        else {
          // Unable to update.
          drupal_set_message(t('Unable to send result ID: @outcome', array('@outcome' => $outcome->lti_tool_provider_outcomes_id), 'error'));
          $bool = FALSE;
        }
      }
    }
    if ($bool) {
      drupal_set_message(t('@total_outcomes outcomes retrieved. @outcomes_counter outcomes sent.', array(
      '@total_outcomes' => $total_outcomes,
      '@outcomes_counter' => $outcomes_counter,
      )));
    }
    lti_tool_provider_goto($return_url);
  }
  else {
    drupal_set_message(t('There is nothing to send.'));
    lti_tool_provider_goto($return_url);
  }
}

/**
 * Retrieves results from the Consumer.
 *
 * @param string $mode
 *   The scope of results to retrieve.
 * @param string $return_url
 *   To customise return page when URL is accessed.
 */
function lti_tool_provider_outcomes_pull_results($mode, $return_url = '') {
  global $user;
  $uid = $user->uid;
  $consumer_id = $_SESSION['lti_tool_provider_context_info']['consumer_id'];
  $context_id = $_SESSION['lti_tool_provider_context_info']['context_id'];
  $result_sourcedid = isset($_SESSION['lti_tool_provider_context_info']['lis_result_sourcedid']) ? $_SESSION['lti_tool_provider_context_info']['lis_result_sourcedid'] : NULL;
  $key = $_SESSION['lti_tool_provider_context_info']['oauth_consumer_key'];
  $resource_link_id = $_SESSION['lti_tool_provider_context_info']['resource_link_id'];
  $bool = TRUE;
  // Get the outcomes to pull.
  switch ($mode) {
    case 'user': {
      // For this user.
      $outcomes_entities = lti_tool_provider_outcomes_get_outcomes($context_id, $resource_link_id, $consumer_id, $uid);
      break;
    }
    // For all results in current resource.
    case 'resource': {
      $outcomes_entities = lti_tool_provider_outcomes_get_outcomes($consumer_id, $context_id, $resource_link_id);
      break;
    }
    // For all results in current context.
    case 'context': {
      $outcomes_entities = lti_tool_provider_outcomes_get_outcomes_for_context($id, $context_id);
      break;
    }
  }
  if (isset($outcomes_entities)) {
    // Construct params.
    $consumer = lti_tool_provider_consumer_load($consumer_id);
    $url = $consumer->lti_tool_provider_consumer_outcomes_url;
    if (!isset($url)) {
      drupal_set_message(t('Unable to send results. No Outcome URL is avaliable', 'error'));
      drupal_goto($return_url);
    }
    else {
      $outcomes_counter = 0;
      $total_outcomes = count($outcomes_entities);
      foreach ($outcomes_entities as $outcome) {
        if ($consumer->lti_tool_provider_consumer_ext) {
          $params = lti_tool_provider_outcomes_construct_params_array_read($outcome->lti_tool_provider_outcomes_result_sourcedid);
          $response = lti_tool_provider_outcomes_lti_service_post($params, $consumer->lti_tool_provider_consumer_key, $consumer->lti_tool_provider_consumer_outcomes_url);
          $score = lti_tool_provider_outcomes_extract_ext_xml_score_reply($response);
          $success = isset($score) ? TRUE : FALSE;
        }
        else {
          $response = lti_tool_provider_outcomes_lti_service_xml($consumer, $consumer->lti_tool_provider_consumer_outcomes_url, 2, $outcome->lti_tool_provider_outcomes_result_sourcedid);
          $score = lti_tool_provider_outcomes_extract_xml_score_reply($response);
          $success = isset($score) ? TRUE : FALSE;
        }
        if ($success) {
          // Once sent, update timestamp.
          lti_tool_provider_outcomes_update_score($outcome, $score);
          $outcomes_counter++;
        }
        else {
          // Unable to update.
          drupal_set_message(t('Unable to retreive result ID: @outcome', array('@outcome' => $outcome->lti_tool_provider_outcomes_id), 'error'));
          $bool = FALSE;
        }
      }
    }
    if ($bool) {
      drupal_set_message(t('@total_outcomes outcomes found. @outcomes_counter outcomes successfully retrieved from consumer.', array(
      '@total_outcomes' => $total_outcomes,
      '@outcomes_counter' => $outcomes_counter,
      )));
    }
    lti_tool_provider_goto($return_url);
  }
  else {
    drupal_set_message(t('Nothing to retrieve.'));
    lti_tool_provider_goto($return_url);
  }
}
